# Importing needed packages
import urllib.request
import pandas as pd
import os
import matplotlib.pyplot as plt
import matplotlib.dates as md
import matplotlib as mpl
import firewxpy.standard as standard
import pytz
import warnings
warnings.filterwarnings('ignore')

from datetime import datetime, timedelta

mpl.rcParams['font.weight'] = 'bold'
mpl.rcParams['xtick.labelsize'] = 7

def sawti(zone_1_threshold_1=10, zone_1_threshold_2=15, zone_1_threshold_3=21, zone_1_threshold_4=40, zone_2_threshold_1=9, zone_2_threshold_2=13, zone_2_threshold_3=20, zone_2_threshold_4=28, zone_3_threshold_1=10, zone_3_threshold_2=16, zone_3_threshold_3=24, zone_3_threshold_4=36, zone_4_threshold_1=9, zone_4_threshold_2=12, zone_4_threshold_3=15, zone_4_threshold_4=25, zone_1_W_weight=1, zone_1_DD_weight=1, zone_1_FMC_weight=1, zone_2_W_weight=1, zone_2_DD_weight=1, zone_2_FMC_weight=1, zone_3_W_weight=1, zone_3_DD_weight=1, zone_3_FMC_weight=1, zone_4_W_weight=1, zone_4_DD_weight=1, zone_4_FMC_weight=1):

    r'''
    This function calculates the The Santa Ana Wildfire Threat Index from Rolinski et al. 2016. The function downloads the .CSV files holding the data, performs the Large Fire Potential (LFP) calculation and makes a bar graph
    for each zone. 

    Literature Citation: Rolinski, T., S. B. Capps, R. G. Fovell, Y. Cao, B. J. D’Agostino, and S. Vanderburg, 2016: The Santa Ana Wildfire Threat Index: Methodology and Operational Implementation. Wea. Forecasting, 31, 1881–1897, https://doi.org/10.1175/WAF-D-15-0141.1.

    Required Arguments: None

    Optional Arguments: 1) zone_1_threshold_1 (Integer) - Default = 10. This is the LFP threshold that seprates no rating (green zone) and a marginal risk (yellow zone) for zone 1. 
                        2) zone_1_threshold_2 (Integer) - Default = 15. This is the LFP threshold that seprates a marginal risk (yellow zone) and a moderate risk (orange zone) for zone 1. 
                        3) zone_1_threshold_3 (Integer) - Default = 21. This is the LFP threshold that seprates a moderate risk (orange zone) and a high risk (red zone) for zone 1. 
                        4) zone_1_threshold_4 (Integer) - Default = 40. This is the LFP threshold that seprates a high risk (red zone) and an extreme risk (purple zone) for zone 1. 
                        5) zone_2_threshold_1 (Integer) - Default = 9. This is the LFP threshold that seprates no rating (green zone) and a marginal risk (yellow zone) for zone 2. 
                        6) zone_2_threshold_2 (Integer) - Default = 13. This is the LFP threshold that seprates a marginal risk (yellow zone) and a moderate risk (orange zone) for zone 2. 
                        7) zone_2_threshold_3 (Integer) - Default = 20. This is the LFP threshold that seprates a moderate risk (orange zone) and a high risk (red zone) for zone 2. 
                        8) zone_2_threshold_4 (Integer) - Default = 28. This is the LFP threshold that seprates a high risk (red zone) and an extreme risk (purple zone) for zone 2. 
                        9) zone_3_threshold_1 (Integer) - Default = 10. This is the LFP threshold that seprates no rating (green zone) and a marginal risk (yellow zone) for zone 3. 
                        10) zone_3_threshold_2 (Integer) - Default = 16. This is the LFP threshold that seprates a marginal risk (yellow zone) and a moderate risk (orange zone) for zone 3. 
                        11) zone_3_threshold_3 (Integer) - Default = 24. This is the LFP threshold that seprates a moderate risk (orange zone) and a high risk (red zone) for zone 3. 
                        12) zone_3_threshold_4 (Integer) - Default = 36. This is the LFP threshold that seprates a high risk (red zone) and an extreme risk (purple zone) for zone 3. 
                        13) zone_4_threshold_1 (Integer) - Default = 9. This is the LFP threshold that seprates no rating (green zone) and a marginal risk (yellow zone) for zone 4. 
                        14) zone_4_threshold_2 (Integer) - Default = 12. This is the LFP threshold that seprates a marginal risk (yellow zone) and a moderate risk (orange zone) for zone 4. 
                        15) zone_4_threshold_3 (Integer) - Default = 15. This is the LFP threshold that seprates a moderate risk (orange zone) and a high risk (red zone) for zone 4. 
                        16) zone_4_threshold_4 (Integer) - Default = 25. This is the LFP threshold that seprates a high risk (red zone) and an extreme risk (purple zone) for zone 4.
                        17) zone_1_W_weight (Float or Integer) - Default = 1. This is the weight to multiply the wind-squared value by for zone 1. 
                        18) zone_1_DD_weight (Float or Integer) - Default = 1. This is the weight to multiply the dew point depression value by for zone 1. 
                        19) zone_1_FMC_weight (Float or Integer) - Default = 1. This is the weight to multiply the fuel moisture component value by for zone 1. 
                        20) zone_2_W_weight (Float or Integer) - Default = 1. This is the weight to multiply the wind-squared value by for zone 2. 
                        21) zone_2_DD_weight (Float or Integer) - Default = 1. This is the weight to multiply the dew point depression value by for zone 2. 
                        22) zone_2_FMC_weight (Float or Integer) - Default = 1. This is the weight to multiply the fuel moisture component value by for zone 2. 
                        23) zone_3_W_weight (Float or Integer) - Default = 1. This is the weight to multiply the wind-squared value by for zone 3. 
                        24) zone_3_DD_weight (Float or Integer) - Default = 1. This is the weight to multiply the dew point depression value by for zone 3. 
                        25) zone_3_FMC_weight (Float or Integer) - Default = 1. This is the weight to multiply the fuel moisture component value by for zone 3. 
                        26) zone_4_W_weight (Float or Integer) - Default = 1. This is the weight to multiply the wind-squared value by for zone 4. 
                        27) zone_4_DD_weight (Float or Integer) - Default = 1. This is the weight to multiply the dew point depression value by for zone 4. 
                        28) zone_4_FMC_weight (Float or Integer) - Default = 1. This is the weight to multiply the fuel moisture component value by for zone 4. 

    Returns: A graphic showing the LFP forecast for each zone saved to f:Weather Data/SAWTI. 
                        
    '''

    local_time, utc_time = standard.plot_creation_time()

    now = utc_time
    yday = now - timedelta(days=1)

    # Sets up folders
    if os.path.exists(f"SAWTI Data"):
        pass
    else:
        os.mkdir(f"SAWTI Data")
    
    # Zone 1

    if os.path.exists(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_00z.csv')
    else:
        pass

    if os.path.exists(f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_00z.csv')
    else:
        pass   

    
    # Zone 2
    if os.path.exists(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_00z.csv')
    else:
        pass

    if os.path.exists(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_00z.csv')
    else:
        pass   
    
    # Zone 3
    if os.path.exists(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_00z.csv')
    else:
        pass

    if os.path.exists(f'SAWTI Data/seaspace_zone3_{yday.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone3_{yday.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone3_{yday.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_00z.csv')
    else:
        pass  

    # Zone 4
    
    if os.path.exists(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_00z.csv')
    else:
        pass

    if os.path.exists(f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_12z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_12z.csv')
    else:
        pass
        
    if os.path.exists(f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_00z.csv'):
        os.remove(f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_00z.csv')
    else:
        pass   
        
    
    '''
    In this section we download the new SAWTI CSV files
    '''
    # Zone 1
    try:
        urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone1-LA-Ventura/seaspace_zone1_'+now.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone1_'+now.strftime('%m%d%Y')+'_12z.csv')
        os.replace('seaspace_zone1_'+now.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_12z.csv')
        today = False
    except Exception as e:
        try:
            urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone1-LA-Ventura/seaspace_zone1_'+now.strftime('%m%d%Y')+'_00z.csv', 'seaspace_zone1_'+now.strftime('%m%d%Y')+'_00z.csv')
            os.replace('seaspace_zone1_'+now.strftime('%m%d%Y')+'_00z.csv', f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_00z.csv')
            today = True
        except Exception as e:
            try:
                urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone1-LA-Ventura/seaspace_zone1_'+yday.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone1_'+yday.strftime('%m%d%Y')+'_12z.csv')
                os.replace('seaspace_zone1_'+yday.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_12z.csv')
                today = False
            except Exception as e:
                pass
    
    # Zone 2
    try:
        urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone2-Orange-InlandEmpire/seaspace_zone2_'+now.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone2_'+now.strftime('%m%d%Y')+'_12z.csv')
        os.replace('seaspace_zone2_'+now.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_12z.csv')
        today = False
    except Exception as e:
        try:
            urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone2-Orange-InlandEmpire/seaspace_zone2_'+now.strftime('%m%d%Y')+'_00z.csv', 'seaspace_zone2_'+now.strftime('%m%d%Y')+'_00z.csv')
            os.replace('seaspace_zone2_'+now.strftime('%m%d%Y')+'_00z.csv', f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_00z.csv')
            today = True
        except Exception as e:
            try:
                urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone2-Orange-InlandEmpire/seaspace_zone2_'+yday.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone2_'+yday.strftime('%m%d%Y')+'_12z.csv')
                os.replace('seaspace_zone2_'+yday.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_12z.csv')
                today = False
            except Exception as e:
                pass
    
    # Zone 3
    try:
        urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone3-SanDiego/seaspace_zone3_'+now.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone3_'+now.strftime('%m%d%Y')+'_12z.csv')
        os.replace('seaspace_zone3_'+now.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_12z.csv')
        today = False
    except Exception as e:
        try:
            urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone3-SanDiego/seaspace_zone3_'+now.strftime('%m%d%Y')+'_00z.csv', 'seaspace_zone3_'+now.strftime('%m%d%Y')+'_00z.csv')
            os.replace('seaspace_zone3_'+now.strftime('%m%d%Y')+'_00z.csv', f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_00z.csv')
            today = True
        except Exception as e:
            try:
                urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone3-SanDiego/seaspace_zone3_'+yday.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone3_'+yday.strftime('%m%d%Y')+'_12z.csv')
                os.replace('seaspace_zone3_'+yday.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone3_{yday.strftime("%m%d%Y")}_12z.csv')
                today = False
            except Exception as e:
                pass
    
    # Zone 4
    try:
        urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone4-SantaBarbara/seaspace_zone4_'+now.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone4_'+now.strftime('%m%d%Y')+'_12z.csv')
        os.replace('seaspace_zone4_'+now.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_12z.csv')
        today = False
    except Exception as e:
        try:
            urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone4-SantaBarbara/seaspace_zone4_'+now.strftime('%m%d%Y')+'_00z.csv', 'seaspace_zone4_'+now.strftime('%m%d%Y')+'_00z.csv')
            os.replace('seaspace_zone4_'+now.strftime('%m%d%Y')+'_00z.csv', f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_00z.csv')
            today = True
        except Exception as e:
            try:
                urllib.request.urlretrieve('https://sdge.sdsc.edu/data/sdge/sawti/Zone4-SantaBarbara/seaspace_zone4_'+yday.strftime('%m%d%Y')+'_12z.csv', 'seaspace_zone4_'+yday.strftime('%m%d%Y')+'_12z.csv')
                os.replace('seaspace_zone4_'+yday.strftime('%m%d%Y')+'_12z.csv', f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_12z.csv')
                if local_time.day == utc_time.day:
                    today = False
                else:
                    today = True
            except Exception as e:
                pass
    
    try:
        df1 = pd.read_csv(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_12z.csv')
        df1 = df1.transpose()
    except Exception as e:
        try:
            df1 = pd.read_csv(f'SAWTI Data/seaspace_zone1_{now.strftime("%m%d%Y")}_00z.csv')
            df1 = df1.transpose()
        except Exception as e:
            try:
                df1 = pd.read_csv(f'SAWTI Data/seaspace_zone1_{yday.strftime("%m%d%Y")}_12z.csv')
                df1 = df1.transpose()
            except Exception as e:
                pass
    
    try:
        df2 = pd.read_csv(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_12z.csv')
        df2 = df2.transpose()
    except Exception as e:
        try:
            df2 = pd.read_csv(f'SAWTI Data/seaspace_zone2_{now.strftime("%m%d%Y")}_00z.csv')
            df2 = df2.transpose()
        except Exception as e:
            try:
                df2 = pd.read_csv(f'SAWTI Data/seaspace_zone2_{yday.strftime("%m%d%Y")}_12z.csv')
                df2 = df2.transpose()
            except Exception as e:
                pass
    
    try:
        df3 = pd.read_csv(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_12z.csv')
        df3 = df3.transpose()
    except Exception as e:
        try:
            df3 = pd.read_csv(f'SAWTI Data/seaspace_zone3_{now.strftime("%m%d%Y")}_00z.csv')
            df3 = df3.transpose()
        except Exception as e:
            try:
                df3 = pd.read_csv(f'SAWTI Data/seaspace_zone3_{yday.strftime("%m%d%Y")}_12z.csv')
                df3 = df3.transpose()
            except Exception as e:
                pass
    
    try:
        df4 = pd.read_csv(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_12z.csv')
        df4 = df4.transpose()
    except Exception as e:
        try:
            df4 = pd.read_csv(f'SAWTI Data/seaspace_zone4_{now.strftime("%m%d%Y")}_00z.csv')
            df4 = df4.transpose()
        except Exception as e:
            try:
                df4 = pd.read_csv(f'SAWTI Data/seaspace_zone4_{yday.strftime("%m%d%Y")}_12z.csv')
                df4 = df4.transpose()
            except Exception as e:
                pass

    day2 = now + timedelta(days=1)
    day3 = now + timedelta(days=2)
    day4 = now + timedelta(days=3)
    day5 = now + timedelta(days=4)
    day6 = now + timedelta(days=5)
    day7 = now + timedelta(days=6)
    day8 = now + timedelta(days=7)

    timezone = pytz.timezone('America/Los_Angeles')
    if today == True:
        now = now.astimezone(timezone)
        day2 = day2.astimezone(timezone)
        day3 = day3.astimezone(timezone)
        day4 = day4.astimezone(timezone)
        day5 = day5.astimezone(timezone)
        day6 = day6.astimezone(timezone)
        day7 = day7.astimezone(timezone)
        
        today_year = now.year
        today_month = now.month
        today_day = now.day
    
        day2_year = day2.year
        day2_month = day2.month
        day2_day = day2.day
    
        day3_year = day3.year
        day3_month = day3.month
        day3_day = day3.day   
    
        day4_year = day4.year
        day4_month = day4.month
        day4_day = day4.day
    
        day5_year = day5.year
        day5_month = day5.month
        day5_day = day5.day
    
        day6_year = day6.year
        day6_month = day6.month
        day6_day = day6.day
    
        day7_year = day7.year
        day7_month = day7.month
        day7_day = day7.day
    
        today_date = datetime(today_year, today_month, today_day)
        day2 = datetime(day2_year, day2_month, day2_day)
        day3 = datetime(day3_year, day3_month, day3_day)
        day4 = datetime(day4_year, day4_month, day4_day)
        day5 = datetime(day5_year, day5_month, day5_day)
        day6 = datetime(day6_year, day6_month, day6_day)
        day7 = datetime(day7_year, day7_month, day7_day)

        dates = []
        dates.append(today_date)
        dates.append(day2)
        dates.append(day3)
        dates.append(day4)
        dates.append(day5)
        dates.append(day6)
        dates.append(day7)

    else:
        day2 = day2.astimezone(timezone)
        day3 = day3.astimezone(timezone)
        day4 = day4.astimezone(timezone)
        day5 = day5.astimezone(timezone)
        day6 = day6.astimezone(timezone)
        day7 = day7.astimezone(timezone)
        day8 = day8.astimezone(timezone)
    
        day2_year = day2.year
        day2_month = day2.month
        day2_day = day2.day
    
        day3_year = day3.year
        day3_month = day3.month
        day3_day = day3.day   
    
        day4_year = day4.year
        day4_month = day4.month
        day4_day = day4.day
    
        day5_year = day5.year
        day5_month = day5.month
        day5_day = day5.day
    
        day6_year = day6.year
        day6_month = day6.month
        day6_day = day6.day
    
        day7_year = day7.year
        day7_month = day7.month
        day7_day = day7.day

        day8_year = day8.year
        day8_month = day8.month
        day8_day = day8.day        

        day2 = datetime(day2_year, day2_month, day2_day)
        day3 = datetime(day3_year, day3_month, day3_day)
        day4 = datetime(day4_year, day4_month, day4_day)
        day5 = datetime(day5_year, day5_month, day5_day)
        day6 = datetime(day6_year, day6_month, day6_day)
        day7 = datetime(day7_year, day7_month, day7_day)        
        day8 = datetime(day8_year, day8_month, day8_day) 

        dates = []
        dates.append(day2)
        dates.append(day3)
        dates.append(day4)
        dates.append(day5)
        dates.append(day6)
        dates.append(day7)
        dates.append(day8)

        
    try:

        z1_FMC_1 = zone_1_FMC_weight * df1['FMC'].iloc[0]
        if z1_FMC_1 > 1:
            z1_FMC_1 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_1 = z1_FMC_1

        z1_FMC_2 = zone_1_FMC_weight * df1['FMC'].iloc[1]
        if z1_FMC_2 > 1:
            z1_FMC_2 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_2 = z1_FMC_2

        z1_FMC_3 = zone_1_FMC_weight * df1['FMC'].iloc[2]
        if z1_FMC_3 > 1:
            z1_FMC_3 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_3 = z1_FMC_3

        z1_FMC_4 = zone_1_FMC_weight * df1['FMC'].iloc[3]
        if z1_FMC_4 > 1:
            z1_FMC_4 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_4 = z1_FMC_4

        z1_FMC_5 = zone_1_FMC_weight * df1['FMC'].iloc[4]
        if z1_FMC_5 > 1:
            z1_FMC_5 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_5 = z1_FMC_5

        z1_FMC_6 = zone_1_FMC_weight * df1['FMC'].iloc[5]
        if z1_FMC_6 > 1:
            z1_FMC_6 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z1_FMC_6 = z1_FMC_6

        z1_FMC_7 = zone_1_FMC_weight * df1['FMC'].iloc[6]
        if z1_FMC_7 > 1:
            z1_FMC_7 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
            print("\n")
        else:
            z1_FMC_7 = z1_FMC_7
        
        LFP1_Day1 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[0]) * (zone_1_DD_weight * df1['DD'].iloc[0]) * (z1_FMC_1), 0)
        LFP1_Day2 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[1]) * (zone_1_DD_weight * df1['DD'].iloc[1]) * (z1_FMC_2), 0)
        LFP1_Day3 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[2]) * (zone_1_DD_weight * df1['DD'].iloc[2]) * (z1_FMC_3), 0)
        LFP1_Day4 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[3]) * (zone_1_DD_weight * df1['DD'].iloc[3]) * (z1_FMC_4), 0)
        LFP1_Day5 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[4]) * (zone_1_DD_weight * df1['DD'].iloc[4]) * (z1_FMC_5), 0)
        LFP1_Day6 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[5]) * (zone_1_DD_weight * df1['DD'].iloc[5]) * (z1_FMC_6), 0)
        LFP1_Day7 = round((0.001) * (zone_1_W_weight * df1['W^2'].iloc[6]) * (zone_1_DD_weight * df1['DD'].iloc[6]) * (z1_FMC_7), 0)
        print("LFP Zone 1 (Day 1 - Date: "+dates[0].strftime('%m/%d/%Y)')+": "+str(LFP1_Day1))
        print("LFP Zone 1 (Day 2 - Date: "+dates[1].strftime('%m/%d/%Y)')+": "+str(LFP1_Day2))
        print("LFP Zone 1 (Day 3 - Date: "+dates[2].strftime('%m/%d/%Y)')+": "+str(LFP1_Day3))
        print("LFP Zone 1 (Day 4 - Date: "+dates[3].strftime('%m/%d/%Y)')+": "+str(LFP1_Day4))
        print("LFP Zone 1 (Day 5 - Date: "+dates[4].strftime('%m/%d/%Y)')+": "+str(LFP1_Day5))
        print("LFP Zone 1 (Day 6 - Date: "+dates[5].strftime('%m/%d/%Y)')+": "+str(LFP1_Day6))
        print("LFP Zone 1 (Day 7 - Date: "+dates[6].strftime('%m/%d/%Y)')+": "+str(LFP1_Day7))
        print('\n')
        zone_1_LFP_values = []
        zone_1_LFP_values.append(LFP1_Day1)
        zone_1_LFP_values.append(LFP1_Day2)
        zone_1_LFP_values.append(LFP1_Day3)
        zone_1_LFP_values.append(LFP1_Day4)
        zone_1_LFP_values.append(LFP1_Day5)
        zone_1_LFP_values.append(LFP1_Day6)
        zone_1_LFP_values.append(LFP1_Day7)
    except Exception as e:
        pass
    
    try:

        z2_FMC_1 = zone_2_FMC_weight * df2['FMC'].iloc[0]
        if z2_FMC_1 > 1:
            z2_FMC_1 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_1 = z2_FMC_1

        z2_FMC_2 = zone_2_FMC_weight * df2['FMC'].iloc[1]
        if z2_FMC_2 > 1:
            z2_FMC_2 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_2 = z2_FMC_2

        z2_FMC_3 = zone_2_FMC_weight * df2['FMC'].iloc[2]
        if z2_FMC_3 > 1:
            z2_FMC_3 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_3 = z2_FMC_3

        z2_FMC_4 = zone_2_FMC_weight * df2['FMC'].iloc[3]
        if z2_FMC_4 > 1:
            z2_FMC_4 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_4 = z2_FMC_4

        z2_FMC_5 = zone_2_FMC_weight * df2['FMC'].iloc[4]
        if z2_FMC_5 > 1:
            z2_FMC_5 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_5 = z2_FMC_5

        z2_FMC_6 = zone_2_FMC_weight * df2['FMC'].iloc[5]
        if z2_FMC_6 > 1:
            z2_FMC_6 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z2_FMC_6 = z2_FMC_6

        z2_FMC_7 = zone_2_FMC_weight * df2['FMC'].iloc[6]
        if z2_FMC_7 > 1:
            z2_FMC_7 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
            print("\n")
        else:
            z2_FMC_7 = z2_FMC_7

        LFP2_Day1 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[0]) * (zone_2_DD_weight * df2['DD'].iloc[0]) * (z2_FMC_1), 0)
        LFP2_Day2 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[1]) * (zone_2_DD_weight * df2['DD'].iloc[1]) * (z2_FMC_2), 0)
        LFP2_Day3 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[2]) * (zone_2_DD_weight * df2['DD'].iloc[2]) * (z2_FMC_3), 0)
        LFP2_Day4 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[3]) * (zone_2_DD_weight * df2['DD'].iloc[3]) * (z2_FMC_4), 0)
        LFP2_Day5 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[4]) * (zone_2_DD_weight * df2['DD'].iloc[4]) * (z2_FMC_5), 0)
        LFP2_Day6 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[5]) * (zone_2_DD_weight * df2['DD'].iloc[5]) * (z2_FMC_6), 0)
        LFP2_Day7 = round((0.001) * (zone_2_W_weight * df2['W^2'].iloc[6]) * (zone_2_DD_weight * df2['DD'].iloc[6]) * (z2_FMC_7), 0)
        print("LFP Zone 2 (Day 1 - Date: "+dates[0].strftime('%m/%d/%Y)')+": "+str(LFP2_Day1))
        print("LFP Zone 2 (Day 2 - Date: "+dates[1].strftime('%m/%d/%Y)')+": "+str(LFP2_Day2))
        print("LFP Zone 2 (Day 3 - Date: "+dates[2].strftime('%m/%d/%Y)')+": "+str(LFP2_Day3))
        print("LFP Zone 2 (Day 4 - Date: "+dates[3].strftime('%m/%d/%Y)')+": "+str(LFP2_Day4))
        print("LFP Zone 2 (Day 5 - Date: "+dates[4].strftime('%m/%d/%Y)')+": "+str(LFP2_Day5))
        print("LFP Zone 2 (Day 6 - Date: "+dates[5].strftime('%m/%d/%Y)')+": "+str(LFP2_Day6))
        print("LFP Zone 2 (Day 7 - Date: "+dates[6].strftime('%m/%d/%Y)')+": "+str(LFP2_Day7))
        print('\n')
        zone_2_LFP_values = []
        zone_2_LFP_values.append(LFP2_Day1)
        zone_2_LFP_values.append(LFP2_Day2)
        zone_2_LFP_values.append(LFP2_Day3)
        zone_2_LFP_values.append(LFP2_Day4)
        zone_2_LFP_values.append(LFP2_Day5)
        zone_2_LFP_values.append(LFP2_Day6)
        zone_2_LFP_values.append(LFP2_Day7)
    except Exception as e:
        pass
    
    try:

        z3_FMC_1 = zone_3_FMC_weight * df3['FMC'].iloc[0]
        if z3_FMC_1 > 1:
            z3_FMC_1 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_1 = z3_FMC_1

        z3_FMC_2 = zone_3_FMC_weight * df3['FMC'].iloc[1]
        if z3_FMC_2 > 1:
            z3_FMC_2 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_2 = z3_FMC_2

        z3_FMC_3 = zone_3_FMC_weight * df3['FMC'].iloc[2]
        if z3_FMC_3 > 1:
            z3_FMC_3 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_3 = z3_FMC_3

        z3_FMC_4 = zone_3_FMC_weight * df3['FMC'].iloc[3]
        if z3_FMC_4 > 1:
            z3_FMC_4 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_4 = z3_FMC_4

        z3_FMC_5 = zone_3_FMC_weight * df3['FMC'].iloc[4]
        if z3_FMC_5 > 1:
            z3_FMC_5 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_5 = z3_FMC_5

        z3_FMC_6 = zone_3_FMC_weight * df3['FMC'].iloc[5]
        if z3_FMC_6 > 1:
            z3_FMC_6 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z3_FMC_6 = z3_FMC_6

        z3_FMC_7 = zone_3_FMC_weight * df3['FMC'].iloc[6]
        if z3_FMC_7 > 1:
            z3_FMC_7 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
            print("\n")
        else:
            z3_FMC_7 = z3_FMC_7
        
        LFP3_Day1 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[0]) * (zone_3_DD_weight * df3['DD'].iloc[0]) * (z3_FMC_1), 0)
        LFP3_Day2 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[1]) * (zone_3_DD_weight * df3['DD'].iloc[1]) * (z3_FMC_2), 0)
        LFP3_Day3 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[2]) * (zone_3_DD_weight * df3['DD'].iloc[2]) * (z3_FMC_3), 0)
        LFP3_Day4 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[3]) * (zone_3_DD_weight * df3['DD'].iloc[3]) * (z3_FMC_4), 0)
        LFP3_Day5 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[4]) * (zone_3_DD_weight * df3['DD'].iloc[4]) * (z3_FMC_5), 0)
        LFP3_Day6 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[5]) * (zone_3_DD_weight * df3['DD'].iloc[5]) * (z3_FMC_6), 0)
        LFP3_Day7 = round((0.001) * (zone_3_W_weight * df3['W^2'].iloc[6]) * (zone_3_DD_weight * df3['DD'].iloc[6]) * (z3_FMC_7), 0)
        print("LFP Zone 3 (Day 1 - Date: "+dates[0].strftime('%m/%d/%Y)')+": "+str(LFP3_Day1))
        print("LFP Zone 3 (Day 2 - Date: "+dates[1].strftime('%m/%d/%Y)')+": "+str(LFP3_Day2))
        print("LFP Zone 3 (Day 3 - Date: "+dates[2].strftime('%m/%d/%Y)')+": "+str(LFP3_Day3))
        print("LFP Zone 3 (Day 4 - Date: "+dates[3].strftime('%m/%d/%Y)')+": "+str(LFP3_Day4))
        print("LFP Zone 3 (Day 5 - Date: "+dates[4].strftime('%m/%d/%Y)')+": "+str(LFP3_Day5))
        print("LFP Zone 3 (Day 6 - Date: "+dates[5].strftime('%m/%d/%Y)')+": "+str(LFP3_Day6))
        print("LFP Zone 3 (Day 7 - Date: "+dates[6].strftime('%m/%d/%Y)')+": "+str(LFP3_Day7))
        print('\n')
        zone_3_LFP_values = []
        zone_3_LFP_values.append(LFP3_Day1)
        zone_3_LFP_values.append(LFP3_Day2)
        zone_3_LFP_values.append(LFP3_Day3)
        zone_3_LFP_values.append(LFP3_Day4)
        zone_3_LFP_values.append(LFP3_Day5)
        zone_3_LFP_values.append(LFP3_Day6)
        zone_3_LFP_values.append(LFP3_Day7)
    except Exception as e:
        pass
    
    try:

        z4_FMC_1 = zone_4_FMC_weight * df4['FMC'].iloc[0]
        if z4_FMC_1 > 1:
            z4_FMC_1 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_1 = z4_FMC_1

        z4_FMC_2 = zone_4_FMC_weight * df4['FMC'].iloc[1]
        if z4_FMC_2 > 1:
            z4_FMC_2 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_2 = z4_FMC_2

        z4_FMC_3 = zone_4_FMC_weight * df4['FMC'].iloc[2]
        if z4_FMC_3 > 1:
            z4_FMC_3 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_3 = z4_FMC_3

        z4_FMC_4 = zone_4_FMC_weight * df4['FMC'].iloc[3]
        if z4_FMC_4 > 1:
            z4_FMC_4 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_4 = z4_FMC_4

        z4_FMC_5 = zone_4_FMC_weight * df4['FMC'].iloc[4]
        if z4_FMC_5 > 1:
            z4_FMC_5 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_5 = z4_FMC_5

        z4_FMC_6 = zone_3_FMC_weight * df3['FMC'].iloc[5]
        if z4_FMC_6 > 1:
            z4_FMC_6 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
        else:
            z4_FMC_6 = z4_FMC_6

        z4_FMC_7 = zone_4_FMC_weight * df4['FMC'].iloc[6]
        if z4_FMC_7 > 1:
            z4_FMC_7 = 1
            print("FMC ranges between 0 and 1. The weight applied puts this value over 1. Setting the FMC value to 1.")
            print("\n")
        else:
            z4_FMC_7 = z4_FMC_7
        
        LFP4_Day1 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[0]) * (zone_4_DD_weight * df4['DD'].iloc[0]) * (z4_FMC_1), 0)
        LFP4_Day2 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[1]) * (zone_4_DD_weight * df4['DD'].iloc[1]) * (z4_FMC_2), 0)
        LFP4_Day3 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[2]) * (zone_4_DD_weight * df4['DD'].iloc[2]) * (z4_FMC_3), 0)
        LFP4_Day4 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[3]) * (zone_4_DD_weight * df4['DD'].iloc[3]) * (z4_FMC_4), 0)
        LFP4_Day5 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[4]) * (zone_4_DD_weight * df4['DD'].iloc[4]) * (z4_FMC_5), 0)
        LFP4_Day6 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[5]) * (zone_4_DD_weight * df4['DD'].iloc[5]) * (z4_FMC_6), 0)
        LFP4_Day7 = round((0.001) * (zone_4_W_weight * df4['W^2'].iloc[6]) * (zone_4_DD_weight * df4['DD'].iloc[6]) * (z4_FMC_7), 0)
        print("LFP Zone 4 (Day 1 - Date: "+dates[0].strftime('%m/%d/%Y)')+": "+str(LFP4_Day1))
        print("LFP Zone 4 (Day 2 - Date: "+dates[1].strftime('%m/%d/%Y)')+": "+str(LFP4_Day2))
        print("LFP Zone 4 (Day 3 - Date: "+dates[2].strftime('%m/%d/%Y)')+": "+str(LFP4_Day3))
        print("LFP Zone 4 (Day 4 - Date: "+dates[3].strftime('%m/%d/%Y)')+": "+str(LFP4_Day4))
        print("LFP Zone 4 (Day 5 - Date: "+dates[4].strftime('%m/%d/%Y)')+": "+str(LFP4_Day5))
        print("LFP Zone 4 (Day 6 - Date: "+dates[5].strftime('%m/%d/%Y)')+": "+str(LFP4_Day6))
        print("LFP Zone 4 (Day 7 - Date: "+dates[6].strftime('%m/%d/%Y)')+": "+str(LFP4_Day7))
        print('\n')
        zone_4_LFP_values = []
        zone_4_LFP_values.append(LFP4_Day1)
        zone_4_LFP_values.append(LFP4_Day2)
        zone_4_LFP_values.append(LFP4_Day3)
        zone_4_LFP_values.append(LFP4_Day4)
        zone_4_LFP_values.append(LFP4_Day5)
        zone_4_LFP_values.append(LFP4_Day6)
        zone_4_LFP_values.append(LFP4_Day7)
    except Exception as e:
        pass


    plt.style.use('seaborn-v0_8-darkgrid')

    print("Creating Image - Please Wait...")
    
    fig = plt.figure(figsize=(12,10))
    fig.set_facecolor('aliceblue')
    fig.suptitle("Santa Ana Wildfire Threat Index (SAWTI)", fontsize=16, fontweight='bold')
    
    ax1 = fig.add_subplot(2, 2, 1)
    ax1.bar(dates, zone_1_LFP_values, color='red', align='center', zorder=2, width=0.4)
    ax1.set_title("Zone 1: Los Angeles & Ventura", fontweight='bold')
    ax1.set_xlabel("Date", fontweight='bold')
    ax1.xaxis.set_major_formatter(md.DateFormatter('%a %m/%d'))
    ax1.set_ylabel("Large Fire Potential", fontweight='bold')
    ax1.set_ylim(0, 60)
    ax1.axhspan(0, zone_1_threshold_1, color='green', alpha=0.3, zorder=1)
    ax1.axhspan(zone_1_threshold_1, zone_1_threshold_2, color='yellow', alpha=0.3, zorder=1)
    ax1.axhspan(zone_1_threshold_2, zone_1_threshold_3, color='orange', alpha=0.3, zorder=1)
    ax1.axhspan(zone_1_threshold_3, zone_1_threshold_4, color='red', alpha=0.3, zorder=1)
    ax1.axhspan(zone_1_threshold_4, 60, color='purple', alpha=0.3, zorder=1)

    ax2 = fig.add_subplot(2, 2, 2)
    ax2.bar(dates, zone_2_LFP_values, color='red', align='center', zorder=2, width=0.4)
    ax2.set_title("Zone 2: Orange County & Inland Empire", fontweight='bold')
    ax2.set_xlabel("Date", fontweight='bold')
    ax2.xaxis.set_major_formatter(md.DateFormatter('%a %m/%d'))
    ax2.set_ylabel("Large Fire Potential", fontweight='bold')
    ax2.set_ylim(0, 60)
    ax2.axhspan(0, zone_2_threshold_1, color='green', alpha=0.3, zorder=1)
    ax2.axhspan(zone_2_threshold_1, zone_2_threshold_2, color='yellow', alpha=0.3, zorder=1)
    ax2.axhspan(zone_2_threshold_2, zone_2_threshold_3, color='orange', alpha=0.3, zorder=1)
    ax2.axhspan(zone_2_threshold_3, zone_2_threshold_4, color='red', alpha=0.3, zorder=1)
    ax2.axhspan(zone_2_threshold_4, 60, color='purple', alpha=0.3, zorder=1)

    ax3 = fig.add_subplot(2, 2, 3)
    ax3.bar(dates, zone_3_LFP_values, color='red', align='center', zorder=2, width=0.4)
    ax3.set_title("Zone 3: San Diego", fontweight='bold')
    ax3.set_xlabel("Date", fontweight='bold')
    ax3.xaxis.set_major_formatter(md.DateFormatter('%a %m/%d'))
    ax3.set_ylabel("Large Fire Potential", fontweight='bold')
    ax3.set_ylim(0, 60)
    ax3.axhspan(0, zone_3_threshold_1, color='green', alpha=0.3, zorder=1)
    ax3.axhspan(zone_3_threshold_1, zone_3_threshold_2, color='yellow', alpha=0.3, zorder=1)
    ax3.axhspan(zone_3_threshold_2, zone_3_threshold_3, color='orange', alpha=0.3, zorder=1)
    ax3.axhspan(zone_3_threshold_3, zone_3_threshold_4, color='red', alpha=0.3, zorder=1)
    ax3.axhspan(zone_3_threshold_4, 60, color='purple', alpha=0.3, zorder=1)

    ax4 = fig.add_subplot(2, 2, 4)
    ax4.bar(dates, zone_4_LFP_values, color='red', align='center', zorder=2, width=0.4)
    ax4.set_title("Zone 4: Santa Barbara", fontweight='bold')
    ax4.set_xlabel("Date", fontweight='bold')
    ax4.xaxis.set_major_formatter(md.DateFormatter('%a %m/%d'))
    ax4.set_ylabel("Large Fire Potential", fontweight='bold')
    ax4.set_ylim(0, 60)
    ax4.axhspan(0, zone_4_threshold_1, color='green', alpha=0.3, zorder=1)
    ax4.axhspan(zone_4_threshold_1, zone_4_threshold_2, color='yellow', alpha=0.3, zorder=1)
    ax4.axhspan(zone_4_threshold_2, zone_4_threshold_3, color='orange', alpha=0.3, zorder=1)
    ax4.axhspan(zone_4_threshold_3, zone_4_threshold_4, color='red', alpha=0.3, zorder=1)
    ax4.axhspan(zone_4_threshold_4, 60, color='purple', alpha=0.3, zorder=1)

    props = dict(boxstyle='round', facecolor='wheat', alpha=1)
    box = dict(boxstyle='round', facecolor='lavender', alpha=1)

    fig.text(0.1, 0.05, "Plot Created With FireWxPy (C) Eric J. Drewitz "+utc_time.strftime('%Y')+"\nData Source: sdge.sdsc.edu\nImage Created: " + local_time.strftime('%m/%d/%Y %H:%M Local') + " (" + utc_time.strftime('%H:%M UTC') + ")", fontsize=12, fontweight='bold', verticalalignment='top', bbox=props, zorder=10)    

    fig.text(0.42, 0.94, "(Rolinski et al. 2016)", fontsize=14, fontweight='roman', verticalalignment='top', zorder=10) 

    fig.text(0.55, 0, f"                                 Weighting of LFP Components By Zone\nZone 1: Wind: {zone_1_W_weight} | Dew Point Depression: {zone_1_DD_weight} | Fuel Moisture Component: {zone_1_FMC_weight}\nZone 2: Wind: {zone_2_W_weight} | Dew Point Depression: {zone_2_DD_weight} | Fuel Moisture Component: {zone_2_FMC_weight}\nZone 3: Wind: {zone_3_W_weight} | Dew Point Depression: {zone_3_DD_weight} | Fuel Moisture Component: {zone_3_FMC_weight}\nZone 4: Wind: {zone_4_W_weight} | Dew Point Depression: {zone_4_DD_weight} | Fuel Moisture Component: {zone_4_FMC_weight}", fontsize=9, fontweight='bold', bbox=box)

    if zone_1_W_weight == 1 and zone_1_DD_weight == 1 and zone_1_FMC_weight == 1 and zone_2_W_weight ==1 and zone_2_DD_weight == 1 and zone_2_FMC_weight == 1 and zone_3_W_weight == 1 and zone_3_DD_weight == 1 and zone_3_FMC_weight == 1 and zone_4_W_weight == 1 and zone_4_DD_weight == 1 and zone_4_FMC_weight == 1:

        fname = 'SAWTI With Normal Weighting.jpg'

    else:

        fname = f"SAWTI With Weights Of {zone_1_W_weight} Zone 1 Wind {zone_1_DD_weight} Zone 1 DD {zone_1_FMC_weight} Zone 1 FMC {zone_2_W_weight} Zone 2 Wind {zone_2_DD_weight} Zone 2 DD {zone_2_FMC_weight} Zone 2 FMC {zone_3_W_weight} Zone 3 Wind {zone_3_DD_weight} Zone 3 DD {zone_3_FMC_weight} Zone 3 FMC {zone_4_W_weight} Zone 4 Wind {zone_4_DD_weight} Zone 4 DD {zone_4_FMC_weight} Zone 4 FMC.jpg"

    if os.path.exists(f"Weather Data"):
        print("Already Satisfied: f:Weather Data Exists.")
        if os.path.exists(f"Weather Data/SAWTI"):
            print("Already Satisfied: f:Weather Data/SAWTI Exists.")
            plt.savefig(f"Weather Data/SAWTI/{fname}", bbox_inches='tight')
            plt.close(fig)
            print(f"Image Saved to f:Weather Data/SAWTI/{fname}")
        else:
            print("f:Weather Data/SAWTI does not exist. Building new branch to the existing directory...")
            os.mkdir(f"Weather Data/SAWTI")
            print("Successfully built new branch.")
            plt.savefig(f"Weather Data/SAWTI/{fname}", bbox_inches='tight')
            plt.close(fig)
            print(f"Image Saved to f:Weather Data/SAWTI/{fname}")
    else:
        print("f:Weather Data does not exist. Building the directory automatically...")
        os.mkdir(f"Weather Data")
        os.mkdir(f"Weather Data/SAWTI")
        print("Successfully built directory and current branch.")
        plt.savefig(f"Weather Data/SAWTI/{fname}", bbox_inches='tight')
        plt.close(fig)
        print(f"Image Saved to f:Weather Data/SAWTI/{fname}")           



import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib.dates as md
import os
import pandas as pd
import shutil

from data_access import FEMS
from raws_sigs import get_psa_percentiles, sort_data_by_zone
from standard import plot_creation_time
from datetime import datetime, timedelta, UTC
from utilities import file_functions

local_time, utc_time = plot_creation_time()

mpl.rcParams['font.weight'] = 'bold'
props = dict(boxstyle='round', facecolor='wheat', alpha=1)

mpl.rcParams['xtick.labelsize'] = 9
mpl.rcParams['ytick.labelsize'] = 9

def create_psa_100hr_fuels_charts(gacc_region, number_of_years_for_averages=15, fuel_model='Y', start_date=None, data=False):
    
    gacc_region = gacc_region.upper()

    path, path_print = file_functions.get_fuels_charts_paths(gacc_region, '100hr Dead Fuel')

    if data == False:
        FEMS.get_raws_sig_data(gacc_region, number_of_years_for_averages, fuel_model, start_date)
        FEMS.get_nfdrs_forecast_data(gacc_region, fuel_model)
    else:
        pass
        
    get_psa_percentiles(gacc_region)
    sort_data_by_zone(gacc_region)

    data_dir = f"FEMS Data/{gacc_region}/PSA Data"
    percentiles_dir = f"FEMS Data/{gacc_region}/PSA Percentiles"

    percentiles = pd.read_csv(f"FEMS Data/{gacc_region}/PSA Percentiles/PSA_Percentiles.csv")

    if start_date == None:
        days = number_of_years_for_averages * 365
        start_date = utc_time - timedelta(days=days)
        start_year = start_date.year
        xmin = start_date
        xmax = utc_time

    else:
        start_date = start_date
        start_year = f"{start_date[0]}{start_date[1]}{start_date[2]}{start_date[3]}"
        start_month = f"{start_date[5]}{start_date[6]}"
        start_day = f"{start_date[8]}{start_date[9]}"
        xmin = datetime(int(start_year), int(start_month), int(start_day))
        xmax = utc_time
    
    psa = 1

    if os.path.exists(f"{data_dir}/.ipynb_checkpoints"):
        shutil.rmtree(f"{data_dir}/.ipynb_checkpoints")
    else:
        pass

    files = os.listdir(f"{data_dir}")
    psa = 1
    for i in range(0, len(files)):

        fname = f"PSA {psa}.png"

        #try:
        df = pd.read_csv(f"{data_dir}/{files[i]}") 

        dates = pd.to_datetime(df['date'])

        fig = plt.figure(figsize=(12,12))

        ax = fig.add_subplot(1, 1, 1)

        ax.xaxis.set_major_formatter(md.DateFormatter('%m-%d'))
        plt.title(f"{gacc_region} 100-HR Dead Fuel Moisture: Zone {psa}", fontsize=12, fontweight='bold', loc='left')
        plt.title(f"Period Of Record: {start_year} - {utc_time.year}", fontsize=10, fontweight='bold', loc='right')

        #ax.hlines(percentiles['100hr_DFM_3_percentile'].iloc[i], xmin=xmin, xmax=xmax, color='black', linestyle='--', alpha=0.5, zorder=2)
        #ax.hlines(percentiles['100hr_DFM_10_percentile'].iloc[i], xmin=xmin, xmax=xmax, color='black', linestyle='--', alpha=0.5, zorder=2)
        #ax.hlines(percentiles['100hr_DFM_20_percentile'].iloc[i], xmin=xmin, xmax=xmax, color='black', linestyle='--', alpha=0.5, zorder=2)
        #ax.hlines(percentiles['100hr_DFM_40_percentile'].iloc[i], xmin=xmin, xmax=xmax, color='black', linestyle='--', alpha=0.5, zorder=2)

        ax.plot(dates, df['100hr_DFM_mean'], color='blue', alpha=1)
        ax.plot(dates, df['100hr_DFM_max'], color='darkgreen', alpha=0.4)
        ax.plot(dates, df['100hr_DFM_min'], color='darkorange', alpha=0.4)
        ax.fill_between(dates, df['100hr_DFM_mean'], df['100hr_DFM_max'], color='green', alpha=0.25, where=(df['100hr_DFM_max'] > df['100hr_DFM_mean']))
        ax.fill_between(dates, df['100hr_DFM_mean'], df['100hr_DFM_min'], color='orange', alpha=0.25, where=(df['100hr_DFM_min'] < df['100hr_DFM_mean']))

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
        psa = psa + 1
        #except Exception as e:
         #   pass

    
    print(f"100-HR Fuels Charts Saved To {path_print}")

        
    


import firewxpy.standard as standard
import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt
import matplotlib.dates as md
import matplotlib as mpl
import numpy as np
import warnings
import os
warnings.filterwarnings('ignore')

from dateutil import tz
from pysolar import solar, radiation
from datetime import datetime, timedelta

def get_solar_noon(times_list, solar_elevation):

    times_list, solar_elevation = times_list, solar_elevation
    max_elevation = np.nanmax(solar_elevation)
    for i, j in zip(times_list, solar_elevation):
        if j == max_elevation:
            return i

def get_solar_midnight(times_list, solar_elevation):

    times_list, solar_elevation = times_list, solar_elevation
    min_elevation = np.nanmin(solar_elevation)
    for i, j in zip(times_list, solar_elevation):
        if j == min_elevation:
            return i 

def plot_daily_solar_information(latitude, longitude):
    
    r'''
    This function plots a daily solar graph and gives other types of solar information. 

    Required Arguments: 1) latitude (Float) - The latitude coordinate in decimal degrees format. 
                        2) longitude (Float) - The longitude coordinate in decimal degrees format. 

    Optional Arguments: None

    Return: A graphic showing the following information saved to f:Weather Data/Solar Information:

    1) Daily solar elevation graphic
    2) A data table showing the following information:
        i) Maximum Daily Solar Elevation (solar-noon sun angle)
        ii) Minimum Daily Solar Elevation (solar-midnight sun angle)
        iii) Difference in daily solar elevation between current day, equinox, summer solstice and winter solstice
        iv) Total Daily Solar Radiation [W/m^2]


    '''


    latitude, longitude = latitude, longitude
    local, utc = standard.plot_creation_time()
    
    from_zone = tz.tzutc()
    to_zone = tz.tzlocal()
    
    start = utc
    year = start.year
    month = start.month
    day = start.day
    
    start = datetime(year, month, day, tzinfo=to_zone)
    if latitude >= 0:
        start_summer_solstice = datetime(year, 6, 21, tzinfo=to_zone)
        start_winter_solstice = datetime(year, 12, 21, tzinfo=to_zone)
    if latitude < 0:
        start_winter_solstice = datetime(year, 6, 21, tzinfo=to_zone)
        start_summer_solstice = datetime(year, 12, 21, tzinfo=to_zone)    
        
    start_equinox = datetime(year, 3, 21, tzinfo=to_zone)
    times_list = [start + timedelta(minutes=i *15) for i in range(24*4)]
    times_list_summer_solstice = [start_summer_solstice + timedelta(minutes=i *15) for i in range(24*4)]
    times_list_winter_solstice = [start_winter_solstice + timedelta(minutes=i *15) for i in range(24*4)]
    times_list_equinox = [start_equinox + timedelta(minutes=i *15) for i in range(24*4)]
    
    solar_elevation = [solar.get_altitude(latitude, longitude, t) for t in times_list]
    solar_radiation = [radiation.get_radiation_direct(t, ele) for t, ele in zip(times_list, solar_elevation)]
    solar_azimuth = [solar.get_azimuth(latitude, longitude, t) for t in times_list]
    
    solar_elevation_summer = [solar.get_altitude(latitude, longitude, t) for t in times_list_summer_solstice]
    solar_radiation_summer = [radiation.get_radiation_direct(t, ele) for t, ele in zip(times_list_summer_solstice, solar_elevation_summer)]
    solar_azimuth_summer = [solar.get_azimuth(latitude, longitude, t) for t in times_list_summer_solstice]
    
    solar_elevation_winter = [solar.get_altitude(latitude, longitude, t) for t in times_list_winter_solstice]
    solar_radiation_winter = [radiation.get_radiation_direct(t, ele) for t, ele in zip(times_list_winter_solstice, solar_elevation_winter)]
    solar_azimuth_winter = [solar.get_azimuth(latitude, longitude, t) for t in times_list_winter_solstice]
    
    solar_elevation_equinox = [solar.get_altitude(latitude, longitude, t) for t in times_list_equinox]
    solar_radiation_equinox = [radiation.get_radiation_direct(t, ele) for t, ele in zip(times_list_equinox, solar_elevation_equinox)]
    solar_azimuth_equinox = [solar.get_azimuth(latitude, longitude, t) for t in times_list_equinox]
    
    
    max_elevation = np.nanmax(solar_elevation)
    min_elevation = np.nanmin(solar_elevation)
    
    max_elevation_summer = np.nanmax(solar_elevation_summer)
    min_elevation_summer = np.nanmin(solar_elevation_summer)
    
    max_elevation_winter = np.nanmax(solar_elevation_winter)
    min_elevation_winter = np.nanmin(solar_elevation_winter)
    
    max_elevation_equinox = np.nanmax(solar_elevation_equinox)
    min_elevation_equinox = np.nanmin(solar_elevation_equinox)
    
    sum_rad = np.nansum(solar_radiation)
    
    diff_ele_ss = max_elevation - max_elevation_summer
    diff_ele_ws = max_elevation - max_elevation_winter
    diff_ele_e = max_elevation - max_elevation_equinox
    
    solar_noon = get_solar_noon(times_list, solar_elevation)
    solar_midnight = get_solar_midnight(times_list, solar_elevation)
    
    plt.style.use('fivethirtyeight')
    props = dict(boxstyle='round', facecolor='wheat', alpha=1)
    props_table = dict(boxstyle='round', facecolor='darkslateblue', alpha=1)

    print("Creating Image - Please Wait...")
    
    fig = plt.figure(figsize=(15,9))
    fig.set_facecolor('silver')
    gs = gridspec.GridSpec(2, 4)
    
    fig.suptitle("Daily Solar Information | Date: " + local.strftime('%m/%d/%Y'), color='black', fontsize=28, fontweight='bold')
    
    if latitude >= 0 and longitude <= 0:
        fig.text(0.315, 0.9, "Latitude: " + str(abs(latitude)) + " (\N{DEGREE SIGN}N) | Longitude: " + str(abs(longitude)) + " (\N{DEGREE SIGN}W)", color='black', fontsize=16, fontweight='bold')
    if latitude >= 0 and longitude > 0:
        fig.text(0.315, 0.9, "Latitude: " + str(abs(latitude)) + " (\N{DEGREE SIGN}N) | Longitude: " + str(abs(longitude)) + " (\N{DEGREE SIGN}E)", color='black', fontsize=16, fontweight='bold')
    if latitude < 0 and longitude <= 0:
        fig.text(0.315, 0.9, "Latitude: " + str(abs(latitude)) + " (\N{DEGREE SIGN}S) | Longitude: " + str(abs(longitude)) + " (\N{DEGREE SIGN}W)", color='black', fontsize=16, fontweight='bold')
    if latitude < 0 and longitude > 0:
        fig.text(0.315, 0.9, "Latitude: " + str(abs(latitude)) + " (\N{DEGREE SIGN}S) | Longitude: " + str(abs(longitude)) + " (\N{DEGREE SIGN}E)", color='black', fontsize=16, fontweight='bold')
    
    ax1 = fig.add_subplot(gs[0:2, 0:4])
    ax1.plot(times_list, solar_elevation, c='orange', label=local.strftime('%b %d'), alpha=1)
    ax1.plot(times_list, solar_elevation_summer, c='red', label='Summer Solstice', alpha=1)
    ax1.plot(times_list, solar_elevation_winter, c='blue', label='Winter Solstice', alpha=1)
    ax1.plot(times_list, solar_elevation_equinox, c='magenta', label='Equinox', alpha=1)
    ax1.axhspan(90, 0, color='gold', alpha=0.25)
    ax1.axhspan(0, -6, color='dodgerblue', alpha=0.25)
    ax1.axhspan(-6, -12, color='blue', alpha=0.25)
    ax1.axhspan(-12, -18, color='midnightblue', alpha=0.25)
    ax1.axhspan(-18, -90, color='black', alpha=0.25)
    ax1.xaxis.set_major_formatter(md.DateFormatter('%H', tz=to_zone))
    ax1.set_xlabel("Hour", color='black', fontsize=12, fontweight='bold')
    ax1.set_ylabel("Solar Elevation Angle (Degrees)", color='black', fontsize=12, fontweight='bold')
    ax1.axhline(0, c='black', linestyle='--', linewidth=2)
    ax1.tick_params(axis='x', colors='black')
    ax1.tick_params(axis='y', colors='black')
    ax1.legend(loc=(0.88,0.9), prop={'size': 8})
    ax1.set_ylim(-90, 90)
    ax1.text(0.01, 0.95, 'DAYTIME (0-90 \N{DEGREE SIGN}ABOVE)', fontsize=7, fontweight='bold', transform=ax1.transAxes, bbox=props)
    ax1.text(0.01, 0.4775, 'CIVIL TWILIGHT (0-6 \N{DEGREE SIGN}BELOW)', fontsize=7, fontweight='bold', transform=ax1.transAxes, bbox=props)
    ax1.text(0.01, 0.445, 'NAUTICAL TWILIGHT (6-12 \N{DEGREE SIGN}BELOW)', fontsize=7, fontweight='bold', transform=ax1.transAxes, bbox=props)
    ax1.text(0.01, 0.41, 'ASTRONOMICAL TWILIGHT (12-18 \N{DEGREE SIGN}BELOW)', fontsize=7, fontweight='bold', transform=ax1.transAxes, bbox=props)
    ax1.text(0.01, 0.3, 'NIGHTIME (18-90 \N{DEGREE SIGN}BELOW)', fontsize=7, fontweight='bold', transform=ax1.transAxes, bbox=props)
    
    if diff_ele_e >= 0:
        sym='+'
    else:
        sym=''
    
    if max_elevation >= 0:
        sym1 = '+'
    else:
        sym1 = ''
    
    if min_elevation >= 0:
        sym2 = '+'
    else:
        sym2 = ''
    
    if max_elevation_equinox >= 0:
        sym3 = '+'
    else:
        sym3 = ''
    
    if max_elevation_winter >= 0:
        sym4 = '+'
    else:
        sym4 = ''
    
    ax1.text(0.3, 0.01, "Maximum Elevation: "+sym1+""+ str(round(max_elevation, 1)) + " [\N{DEGREE SIGN} From Horizon] @ "+solar_noon.strftime('%H:%M Local')+"\n\nMinimum Elevation: "+sym2+""+ str(round(min_elevation, 1)) + " [\N{DEGREE SIGN} From Horizon] @ "+solar_midnight.strftime('%H:%M Local')+"\n\nMaximum Elevation Difference:\nSummer Solstice: "+str(round(diff_ele_ss,1))+" [\N{DEGREE SIGN}] | Max Elevation: "+sym1+""+str(round(max_elevation_summer,1))+" [\N{DEGREE SIGN} From Horizon]\nEquinox: "+sym+""+str(round(diff_ele_e,1))+" [\N{DEGREE SIGN}] | Max Elevation: "+sym3+""+str(round(max_elevation_equinox,1))+" [\N{DEGREE SIGN} From Horizon]\nWinter Solstice: "+str(round(diff_ele_ws,1))+" [\N{DEGREE SIGN}] | Max Elevation: "+sym4+""+str(round(max_elevation_winter,1))+" [\N{DEGREE SIGN} From Horizon]\n\nTotal Daily Solar Radiation: " + str(round(sum_rad, 1)) + " (W/m^2)", fontsize=10, fontweight='bold', color='white', transform=ax1.transAxes, bbox=props_table)


    fig.text(0.08, 0.04, "Plot Created With FireWxPy (C) Eric J. Drewitz "+utc_time.strftime('%Y')+"\nImage Created: " + local.strftime('%m/%d/%Y %H:%M Local') + " (" + utc.strftime('%H:%M UTC') + ")", fontsize=10, fontweight='bold', verticalalignment='top', bbox=props, zorder=10)    
    
    if latitude >= 0:
        lat_char = 'N'
    if latitude < 0:
        lat_char = 'S'
    if longitude >= 0:
        lon_char = 'E'
    if longitude < 0:
        lon_char = 'W'
    
    lat_name = str(abs(latitude))
    lon_name = str(abs(longitude))
    
    fname = f"{lat_name}{lat_char}_{lon_name}{lon_char}_SolarInfo.png"

    path = f"Weather Data/Solar Information/{fname}"

    if os.path.exists(f"Weather Data"):
        print("Already Satisfied: f:Weather Data exists.")
        
        if os.path.exists(f"Weather Data/Solar Information"):
            print("Already Satisfied: f:Weather Data/Solar Information exists.")
            fig.savefig(path)
            plt.close(fig)
            print(f"Image saved to: {path}")
            

        else:
            print("f:Weather Data does not exist. Building the new branch...")
            os.mkdir(f"Weather Data/Solar Information")
            print("Successfully built new branch")
            fig.savefig(path)
            plt.close(fig)
            print(f"Image saved to: {path}")            

    else:
        print("f:Weather Data does not exist. Building the new directory...")
        os.mkdir(f"Weather Data")
        os.mkdir(f"Weather Data/Solar Information")
        print("Successfully built directory.")
        fig.savefig(path)
        plt.close(fig)
        print(f"Image saved to: {path}")        
             
        
